 # -*- coding: utf-8 -*-
from setuphelpers import *

# registry key(s) where WAPT will find how to remove the application(s)
uninstallkey = []



def install():

    def get_vers_winff(soft):
        if isfile(makepath(install_location(soft['key']),'winff.exe')):
            return get_file_properties(makepath(install_location(soft['key']),'winff.exe'))['FileVersion']

    if iswin64():
        install_exe_if_needed('WinFF-1.5.5-setup.exe',silentflags='/VERYSILENT /NORESTART',key='WinFF_is1',min_version='1.5.5.0',get_version=get_vers_winff)
    else:
        install_exe_if_needed('WinFF-1.5.4-setup.exe',silentflags='/VERYSILENT /NORESTART',key='WinFF_is1',min_version='1.5.4.0',get_version=get_vers_winff)






# re def install_exe_if_needed because bug in 1.3.9.3

def install_exe_if_needed(exe,silentflags=None,key=None,min_version=None,killbefore=[],accept_returncodes=[0,1603,3010],timeout=300,get_version=None):
    """Install silently the supplied setup executable file, and add the uninstall key to
    global uninstallkey list if it is defined.

    Check if already installed at the supllied min_version.

    Kill the processes in killbefore list before launching the setup.

    Raises an error if, after the setup install, the uninstall key is not found in registry.

    Args:
        exe (str) : path to the setup exe file
        silentflags (str) : flags to append to the exe command line for silent install
                            if not provided, tries to guess them.
        key (str) : uninstall key to check in registry and to add to uninstallkey global list
        min_version (str) : if installed version is equal or gretaer than this, don't install
                            if not provided (None), guess it from exe setup file properties.
                            if == '': ignore version check.
        kill_before (list of str) : processes to kill before setup, to avoid file locks
                                    issues.
        get_version (callable) : optional func to get installed software version from one entry retunred by installed_softwares
            if not provided, version is taken from 'version' attribute in uninstall registry
    Returns:
        None

    """
    if not isfile(exe):
        error('setup exe file %s not found in package' % exe)
    if silentflags is None:
        silentflags = getsilentflags(exe)
    # use empty string to ignore version checking
    if min_version is None:
        min_version = getproductprops(exe)['version']

    import inspect
    caller_globals = inspect.stack()[1][0].f_globals
    WAPT = caller_globals.get('WAPT',None)
    force = WAPT and WAPT.options.force

    if need_install(key,min_version=min_version or None,force=force,get_version=get_version):
        if killbefore:
            killalltasks(killbefore)
        run(r'"%s" %s' % (exe,silentflags),accept_returncodes=accept_returncodes,timeout=timeout)
        if key and not installed_softwares(uninstallkey=key):
            error('Setup %s has been ran but the uninstall key %s can not be found' % (exe,key))
        if key and min_version is not None and need_install(key,min_version=min_version or None,force=False,get_version=get_version):
            error('Setup %s has been and uninstall key %s found but version is not good' % (exe,key))
    else:
        print('Exe setup %s already installed. Skipping' % exe)
    if key:
        # will try to add key in the caller's (setup.py) uninstallkey list
        if 'uninstallkey' in caller_globals and not key in caller_globals['uninstallkey']:
            caller_globals['uninstallkey'].append(key)

